==> gradio_i18n-0.0.2.tar.gz: gradio_i18n-0.0.2/src/gradio_i18n/i18n.py <==
import inspect

import gradio as gr
from gradio.blocks import Block, BlockContext


class I18nString(str):
    def __str__(self):
        return self


def gettext(key: str):
    """Wrapper text string to return I18nString"""
    return I18nString(key)


def iter_i18n_fields(component: gr.components.Component):
    for name, value in inspect.getmembers(component):
        if isinstance(value, I18nString):
            yield name


def iter_i18n_components(block: Block):
    if isinstance(block, BlockContext):
        for component in block.children:
            for c in iter_i18n_components(component):
                yield c

    if any(iter_i18n_fields(block)):
        yield block


def dump_blocks(block: Block, langs=["en"], include_translations={}):
    """Dump all I18nStrings in the block to a dictionary"""
    components = list(iter_i18n_components(block))

    def get(lang, key):
        return include_translations.get(lang, {}).get(key, key)

    ret = {}

    for lang in langs:
        ret[lang] = {}
        for component in components:
            for field in iter_i18n_fields(component):
                value = "" + getattr(component, field)
                ret[lang][value] = get(lang, value)

    return ret


def translate_blocks(block: gr.Blocks, translation={}, state: gr.State = None):
    """Translate all I18nStrings in the block"""
    if not isinstance(block, gr.Blocks):
        raise ValueError("block must be an instance of gradio.Blocks")

    components = list(iter_i18n_components(block))

    def get(lang, key):
        return translation.get(lang, {}).get(key, key)

    def set_lang(request: gr.Request, state):
        if state is None:
            state = {}
        try:
            lang = (
                request.headers["Accept-Language"].split(",")[0].split("-")[0].lower()
            )
            if not lang:
                lang = "en"
                return
        finally:
            if isinstance(state, dict):
                state["lang"] = lang

        outputs = [state]
        for component in components:
            fields = list(iter_i18n_fields(component))

            modified = {field: get(lang, getattr(component, field)) for field in fields}
            new_comp = gr.update(**modified)
            outputs.append(new_comp)

        if len(outputs) == 1:
            return outputs[0]

        return outputs

    if state is None:
        state = gr.State()

    block.load(set_lang, inputs=[state], outputs=components + [state])

==> gradio_i18n-0.0.4.tar.gz: gradio_i18n-0.0.4/src/gradio_i18n/i18n.py <==
import inspect

import gradio as gr
from gradio.blocks import Block, BlockContext, Context


class I18nString(str):
    def __str__(self):
        return self


def gettext(key: str):
    """Wrapper text string to return I18nString"""
    return I18nString(key)


def iter_i18n_fields(component: gr.components.Component):
    for name, value in inspect.getmembers(component):
        if isinstance(value, I18nString):
            yield name


def iter_i18n_components(block: Block):
    if isinstance(block, BlockContext):
        for component in block.children:
            for c in iter_i18n_components(component):
                yield c

    if any(iter_i18n_fields(block)):
        yield block


def dump_blocks(block: Block, langs=["en"], include_translations={}):
    """Dump all I18nStrings in the block to a dictionary"""
    components = list(iter_i18n_components(block))

    def get(lang, key):
        return include_translations.get(lang, {}).get(key, key)

    ret = {}

    for lang in langs:
        ret[lang] = {}
        for component in components:
            for field in iter_i18n_fields(component):
                value = "" + getattr(component, field)
                ret[lang][value] = get(lang, value)

    return ret


def translate_blocks(
    block: gr.Blocks = None, translation={}, lang_state: gr.State = None
):
    if block is None:
        block = Context.root_block

    """Translate all I18nStrings in the block"""
    if not isinstance(block, gr.Blocks):
        raise ValueError("block must be an instance of gradio.Blocks")

    components = list(iter_i18n_components(block))

    def get(lang, key):
        return translation.get(lang, {}).get(key, key)

    def set_lang(request: gr.Request):
        lang = request.headers["Accept-Language"].split(",")[0].split("-")[0].lower()
        if not lang:
            return

        outputs = [lang]
        for component in components:
            fields = list(iter_i18n_fields(component))

            modified = {field: get(lang, getattr(component, field)) for field in fields}
            new_comp = gr.update(**modified)
            outputs.append(new_comp)

        if len(outputs) == 1:
            return outputs[0]

        return outputs

    if lang_state is None:
        lang_state = gr.State()

    block.load(set_lang, outputs=[lang_state] + components)
